<?php
/*
Plugin Name: Info Blocks for WooCommerce
Plugin URI: https://wpfactory.com/item/info-blocks-for-woocommerce/
Description: Info blocks for WooCommerce.
Version: 1.0.1
Author: Algoritmika Ltd
Author URI: http://algoritmika.com
Text Domain: info-blocks-for-woocommerce
Domain Path: /langs
Copyright:  2019 Algoritmika Ltd
WC tested up to: 3.5
License: GNU General Public License v3.0
License URI: http://www.gnu.org/licenses/gpl-3.0.html
*/

if ( ! defined( 'ABSPATH' ) ) exit; // Exit if accessed directly

// Check if WooCommerce is active
$plugin = 'woocommerce/woocommerce.php';
if (
	! in_array( $plugin, apply_filters( 'active_plugins', get_option( 'active_plugins', array() ) ) ) &&
	! ( is_multisite() && array_key_exists( $plugin, get_site_option( 'active_sitewide_plugins', array() ) ) )
) {
	return;
}

if ( 'info-blocks-for-woocommerce.php' === basename( __FILE__ ) ) {
	// Check if Pro is active, if so then return
	$plugin = 'info-blocks-for-woocommerce-pro/info-blocks-for-woocommerce-pro.php';
	if (
		in_array( $plugin, apply_filters( 'active_plugins', get_option( 'active_plugins', array() ) ) ) ||
		( is_multisite() && array_key_exists( $plugin, get_site_option( 'active_sitewide_plugins', array() ) ) )
	) {
		return;
	}
}

if ( ! class_exists( 'Alg_WC_Info_Blocks' ) ) :

/**
 * Main Alg_WC_Info_Blocks Class
 *
 * @class   Alg_WC_Info_Blocks
 * @version 1.0.0
 * @since   1.0.0
 */
final class Alg_WC_Info_Blocks {

	/**
	 * Plugin version.
	 *
	 * @var   string
	 * @since 1.0.0
	 */
	public $version = '1.0.1';

	/**
	 * @var   Alg_WC_Info_Blocks The single instance of the class
	 * @since 1.0.0
	 */
	protected static $_instance = null;

	/**
	 * Main Alg_WC_Info_Blocks Instance
	 *
	 * Ensures only one instance of Alg_WC_Info_Blocks is loaded or can be loaded.
	 *
	 * @version 1.0.0
	 * @since   1.0.0
	 * @static
	 * @return  Alg_WC_Info_Blocks - Main instance
	 */
	public static function instance() {
		if ( is_null( self::$_instance ) ) {
			self::$_instance = new self();
		}
		return self::$_instance;
	}

	/**
	 * Alg_WC_Info_Blocks Constructor.
	 *
	 * @version 1.0.0
	 * @since   1.0.0
	 * @access  public
	 */
	function __construct() {

		// Set up localisation
		load_plugin_textdomain( 'info-blocks-for-woocommerce', false, dirname( plugin_basename( __FILE__ ) ) . '/langs/' );

		// Include required files
		$this->includes();

		// Admin
		if ( is_admin() ) {
			$this->admin();
		}

	}

	/**
	 * action_links.
	 *
	 * @version 1.0.0
	 * @since   1.0.0
	 * @param   mixed $links
	 * @return  array
	 * @todo    [dev] clean up or re-enable "Settings"
	 */
	function action_links( $links ) {
		$custom_links = array();
//		$custom_links[] = '<a href="' . admin_url( 'admin.php?page=wc-settings&tab=alg_wc_info_blocks' ) . '">' . __( 'Settings', 'woocommerce' ) . '</a>';
		$custom_links[] = '<a href="' . admin_url( 'edit.php?post_type=alg_wc_info_block' ) . '">' . __( 'Blocks', 'woocommerce' ) . '</a>';
		if ( 'info-blocks-for-woocommerce.php' === basename( __FILE__ ) ) {
			$custom_links[] = '<a target="_blank" href="https://wpfactory.com/item/info-blocks-for-woocommerce/">' .
				__( 'Unlock all', 'info-blocks-for-woocommerce' ) . '</a>';
		}
		return array_merge( $custom_links, $links );
	}

	/**
	 * includes.
	 *
	 * @version 1.0.0
	 * @since   1.0.0
	 */
	function includes() {
		// Core
		$this->core = require_once( 'includes/class-alg-wc-info-blocks-core.php' );
	}

	/**
	 * admin.
	 *
	 * @version 1.0.0
	 * @since   1.0.0
	 */
	function admin() {
		// Action links
		add_filter( 'plugin_action_links_' . plugin_basename( __FILE__ ), array( $this, 'action_links' ) );
		// Meta boxes
		require_once( 'includes/settings/class-alg-wc-info-blocks-meta-boxes.php' );
		// Settings
		/* add_filter( 'woocommerce_get_settings_pages', array( $this, 'add_woocommerce_settings_tab' ) );
		require_once( 'includes/settings/class-alg-wc-info-blocks-settings-section.php' );
		$this->settings = array();
		$this->settings['general'] = require_once( 'includes/settings/class-alg-wc-info-blocks-settings-general.php' ); */
		// Version updated
		if ( get_option( 'alg_wc_info_blocks_version', '' ) !== $this->version ) {
			add_action( 'admin_init', array( $this, 'version_updated' ) );
		}
	}

	/**
	 * version_updated.
	 *
	 * @version 1.0.0
	 * @since   1.0.0
	 */
	function version_updated() {
		update_option( 'alg_wc_info_blocks_version', $this->version );
	}

	/**
	 * add_woocommerce_settings_tab.
	 *
	 * @version 1.0.0
	 * @since   1.0.0
	 */
	function add_woocommerce_settings_tab( $settings ) {
		$settings[] = require_once( 'includes/settings/class-alg-wc-info-blocks-settings.php' );
		return $settings;
	}

	/**
	 * plugin_url.
	 *
	 * @version 1.0.0
	 * @since   1.0.0
	 * @return  string
	 */
	function plugin_url() {
		return untrailingslashit( plugin_dir_url( __FILE__ ) );
	}

	/**
	 * plugin_path.
	 *
	 * @version 1.0.0
	 * @since   1.0.0
	 * @return  string
	 */
	function plugin_path() {
		return untrailingslashit( plugin_dir_path( __FILE__ ) );
	}

}

endif;

if ( ! function_exists( 'alg_wc_info_blocks' ) ) {
	/**
	 * Returns the main instance of Alg_WC_Info_Blocks to prevent the need to use globals.
	 *
	 * @version 1.0.0
	 * @since   1.0.0
	 * @return  Alg_WC_Info_Blocks
	 */
	function alg_wc_info_blocks() {
		return Alg_WC_Info_Blocks::instance();
	}
}

alg_wc_info_blocks();
